'use strict';


function inputArea() {
	return $('div#input > textarea#input');
}

function inputFile() {
	return $('div#input > input#input');
}

function inputButton() {
	return $('div#input > button#compile');
}

function minicArea() {
	return $('div#minic > iframe#minic');
}

function assemblyArea() {
	return $('div#assembly > iframe#assembly');
}

function rulesArea() {
	return $('div#rules > iframe#rules');
}


$(document).ready(function() {
	inputFile().change(function(event) {
		var reader = new FileReader();

		reader.onload = function(e) {
			inputArea().val(reader.result);
		};

		reader.readAsText($(this)[0].files[0], 'UTF-8');
	});

	// Files remain selected after refresh, so let's pretend it's been
	// selected just now
	if (inputFile()[0].files[0] !== undefined && inputArea().val() === '') {
		inputFile().change();
	}


	inputButton().click(function(event) {
		$.ajax({
			type: 'POST',
			url: '/',
			dataType: 'json',
			contentType: 'application/json; charset=UTF-8',
			data: JSON.stringify({
				code: inputArea().val()
			})
		}).done(function (msg) {
			if (msg.error === true) {
				minicArea().attr('srcdoc', msg.errormsg);
				assemblyArea().attr('srcdoc', '');
				rulesArea().attr('srcdoc', '');
			} else {
				minicArea().attr('srcdoc', msg.minic);
				assemblyArea().attr('srcdoc', msg.assembly);
				rulesArea().attr('srcdoc', msg.rules);
			}
		}).fail(function (req, status, error) {
			alert('Submission failed!\n\n'
				+ 'Req: ' + JSON.stringify(req)
				+ ',\nStatus: ' + status
				+ ',\nError: ' + error);
		});
	});


	$(document).delegate('div#input > textarea#input', 'keydown', function (e) {
		var keyCode = e.keyCode || e.which;

		if (keyCode !== 9)
			return;

		e.preventDefault();
		var start = this.selectionStart;
		var end   = this.selectionEnd;

		$(this).val(
			$(this).val().substring(0, start) + '\t' + $(this).val().substring(end)
		);

		this.selectionStart = this.selectionEnd = start + 1;
	});
});
