'use strict';

require('babel-register');
var cp   = require('child_process');
var fs   = require('fs');
var http = require('http');
var url  = require('url');
var tmp  = require('tmp');
var path = require('path');


function response404(response) {
	response.writeHead(404, {
		'Content-Type': 'text/html; charset=UTF-8'
	});
	fs.createReadStream('404.html').pipe(response);
}

function response500(response) {
	response.writeHead(500, {
		'Content-Type': 'text/html; charset=UTF-8'
	});
	fs.createReadStream('500.html').pipe(response);
}

function checkFileExists(path) {
	try {
		fs.statSync(path);
		return true;
	} catch (e) {
		console.log(`Does not exist: ${path}`);
		return false;
	}
}


tmp.setGracefulCleanup();


http.createServer(function(request, response) {
	if (request.method === 'GET') {
		var name = url.parse(request.url).pathname;

		if (name === '/')
			name = '/index.html';

		name = `.${name}`;
		console.log(`GET: ${name}`);

		var htmlRE    = /\.(html|htm)$/i;
		var cssRE     = /\.css$/i;
		var jsRE      = /\.js$/i;
		var faviconRE = /^\.\/favicon\.ico$/;

		if (checkFileExists(name)) {
			if (htmlRE.test(name)) {
				console.log(`Sending html: ${name}`);
				response.writeHead(200, {
					'Content-Type': 'text/html; charset=UTF-8'
				});
			} else if (cssRE.test(name)) {
				console.log(`Sending css: ${name}`);
				response.writeHead(200, {
					'Content-Type': 'text/css; charset=UTF-8'
				});
			} else if (jsRE.test(name)) {
				console.log(`Sending js: ${name}`);
				response.writeHead(200, {
					'Content-Type': 'text/javascript; charset=UTF-8'
				});
			} else {
				console.log('Sending favicon');
				response.writeHead(200, {
					'Content-Type': 'image/x-icon'
				});
			}

			fs.createReadStream(name).pipe(response);
		} else {
			console.log('Sending FILE NOT FOUND');
			response404(response);
		}
	} else if (request.method === 'POST') {
		response.writeHead(200, {
				'Content-Type': 'application/json; charset=UTF-8'
			});

		var data = '';
		request.on('data', chunk => data += chunk);

		request.on('end', () => {
			var dir      = tmp.dirSync({ unsafeCleanup: true });
			__dirname = __dirname.replace(/ /g, '\\ ');
			var received = path.join(__dirname, 'received.mc');
			var minic    = path.join(__dirname, 'color_mc.html');
			var assembly = path.join(__dirname, 'output.html');
			var rules    = path.join(__dirname, 'rules.html');
			var exe      = path.join(__dirname, 'micko');
			console.log(`Acquired directory: ${dir.name}`);
			console.log(`Received JSON: ${data}`);

			fs.writeFile(received, JSON.parse(data).code, _ => {
				cp.exec(`${exe} < ${received}`, { cwd: __dirname }, (error, stdout, stderr) => {
					if (error !== null) {
						response.write(JSON.stringify({
							error: true,
							errormsg: `${stderr}<br>${stdout}`
						}));
						response.end();
						console.log('Sending unsuccessful JSON');
					} else {
						fs.readFile(minic, 'UTF-8', (_, data_minic) => {
						fs.readFile(assembly, 'UTF-8', (_, data_assembly) => {
						fs.readFile(rules, 'UTF-8', (_, data_rules) => {
							response.write(JSON.stringify({
								error: false,
								errormsg: '',
								minic: data_minic,
								assembly: data_assembly,
								rules: data_rules
							}));
							response.end();
							console.log('Sending successful JSON');
						})})});
					}
				});
			});
		})
	} else {
		console.log(`Sending NOT IMPLEMENTED for ${request.method}`);
		response500(response);
	}
}).listen(8080);
